package com.hz.pm.api.datascope.provider.impl;

import com.ningdatech.basic.util.CollUtils;
import com.hz.pm.api.common.helper.RegionCacheHelper;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.provider.DataScopeProvider;
import com.hz.pm.api.sys.model.dto.RegionDTO;
import com.hz.pm.api.user.entity.enumeration.RoleEnum;
import com.hz.pm.api.user.security.auth.model.UserFullInfoDTO;
import com.hz.pm.api.user.security.auth.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 * ExpertAdminDataScopeProviderImpl
 * </p>
 *
 * @author WendyYang
 * @since 10:39 2023/12/11
 */
@Component("EXPERT_ADMIN")
@RequiredArgsConstructor
public class ExpertAdminDataScopeProviderImpl implements DataScopeProvider {

    private final RegionCacheHelper regionCacheHelper;
    private final UserInfoHelper userInfoHelper;

    @Override
    public Optional<DataScopeDTO> findDataFieldProperty() {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        DataScopeDTO ds = new DataScopeDTO();
        if (Objects.isNull(user.getRegionLevel())) {
            ds.setRegionIds(Collections.emptyList());
            ds.setRegionCodes(Collections.emptyList());
        } else {
            List<RegionDTO> regions = regionCacheHelper.listChildren(user.getRegionCode(), user.getRegionLevel());
            ds.setRegionCodes(CollUtils.fieldList(regions, RegionDTO::getRegionCode));
            ds.setRegionIds(CollUtils.fieldList(regions, RegionDTO::getId));
        }
        ds.setRole(RoleEnum.EXPERT_ADMIN);
        ds.setUserId(user.getUserId());
        ds.setEmployeeCode(user.getEmployeeCode());
        ds.setOrgCode(user.getOrganizationCode());
        return Optional.of(ds);
    }

    @Override
    public Optional<DataScopeDTO> findDataFieldProperty(Long userId) {
        UserFullInfoDTO user = userInfoHelper.getUserFullInfo(userId);
        DataScopeDTO ds = new DataScopeDTO();
        if (Objects.isNull(user.getRegionLevel())) {
            ds.setRegionIds(Collections.emptyList());
            ds.setRegionCodes(Collections.emptyList());
        } else {
            List<RegionDTO> regions = regionCacheHelper.listChildren(user.getRegionCode(), user.getRegionLevel());
            ds.setRegionCodes(CollUtils.fieldList(regions, RegionDTO::getRegionCode));
            ds.setRegionIds(CollUtils.fieldList(regions, RegionDTO::getId));
        }
        ds.setRole(RoleEnum.EXPERT_ADMIN);
        ds.setUserId(user.getUserId());
        ds.setEmployeeCode(user.getEmployeeCode());
        ds.setOrgCode(user.getOrganizationCode());
        return Optional.of(ds);
    }

}
