package com.hz.pm.api.datascope.provider.impl;

import cn.hutool.core.collection.CollUtil;
import com.google.common.collect.Lists;
import com.hz.pm.api.common.constant.RegionConst;
import com.hz.pm.api.common.helper.RegionCacheHelper;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.provider.DataScopeProvider;
import com.hz.pm.api.user.entity.enumeration.RoleEnum;
import com.hz.pm.api.user.security.auth.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Optional;

/**
 * <p>
 * 数据权限
 * </p>
 *
 * @author ZPF
 * @since 2023/3/27 23:29
 */
@Slf4j
@RequiredArgsConstructor
@Component("HIGH_MEMBER")
public class HigherUserDataScopeProviderImpl implements DataScopeProvider {

    private final RegionCacheHelper regionCacheHelper;
    private final UserInfoHelper userInfoHelper;

    //和区域管理员一样
    @Override
    public Optional<DataScopeDTO> findDataFieldProperty() {
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        //借由 丽水市的code 来映射市本级的regionCode
        Collection<String> regionCodes = regionCacheHelper.listChildRegionCodeList(userFullInfo.getRegionCode(),
                RegionConst.RL_CITY);
        //如果不是市本级的话 就直接放入本区域的县就好了
        if(CollUtil.isEmpty(regionCodes)){
            regionCodes = Lists.newArrayList();
            regionCodes.add(userFullInfo.getRegionCode());
        }

        DataScopeDTO ds = new DataScopeDTO();
        ds.setRegionCodes(new ArrayList<>(regionCodes));
        ds.setUserId(userFullInfo.getUserId());
        ds.setEmployeeCode(userFullInfo.getEmployeeCode());
        ds.setRole(RoleEnum.REGION_MANAGER);
        return Optional.of(ds);
    }

    @Override
    public Optional<DataScopeDTO> findDataFieldProperty(Long userId) {
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        //借由 丽水市的code 来映射市本级的regionCode
        Collection<String> regionCodes = regionCacheHelper.listChildRegionCodeList(userFullInfo.getRegionCode(),
                RegionConst.RL_CITY);
        //如果不是市本级的话 就直接放入本区域的县就好了
        if(CollUtil.isEmpty(regionCodes)){
            regionCodes = Lists.newArrayList();
            regionCodes.add(userFullInfo.getRegionCode());
        }

        DataScopeDTO ds = new DataScopeDTO();
        ds.setRegionCodes(new ArrayList<>(regionCodes));
        ds.setUserId(userFullInfo.getUserId());
        ds.setEmployeeCode(userFullInfo.getEmployeeCode());
        ds.setRole(RoleEnum.REGION_MANAGER);
        return Optional.of(ds);
    }
}
