package com.hz.pm.api.datascope.provider.impl;

import com.hz.pm.api.common.constant.RegionConst;
import com.hz.pm.api.common.helper.RegionCacheHelper;
import com.hz.pm.api.common.helper.UserInfoHelper;
import com.hz.pm.api.datascope.model.DataScopeDTO;
import com.hz.pm.api.datascope.provider.DataScopeProvider;
import com.hz.pm.api.user.entity.enumeration.RoleEnum;
import com.hz.pm.api.user.security.auth.model.UserFullInfoDTO;
import com.hz.pm.api.user.util.LoginUserUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Optional;

/**
 * <p>
 * 数据权限
 * </p>
 *
 * @author ZPF
 * @since 2023/3/27 23:29
 */
@Slf4j
@RequiredArgsConstructor
@Component("SUPER_ADMIN")
public class SuperAdminDataScopeProviderImpl implements DataScopeProvider {

    private final RegionCacheHelper regionCacheHelper;

    private final UserInfoHelper userInfoHelper;

    @Override
    public Optional<DataScopeDTO> findDataFieldProperty() {
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(LoginUserUtil.getUserId());
        //所有 丽水市的code
        Collection<String> regionCodes = regionCacheHelper.listChildRegionCodeList(RegionConst.RC_LS,
                RegionConst.RL_CITY);

        DataScopeDTO ds = new DataScopeDTO();
        ds.setRegionCodes(new ArrayList<>(regionCodes));
        ds.setUserId(userFullInfo.getUserId());
        ds.setEmployeeCode(userFullInfo.getEmployeeCode());
        ds.setRole(RoleEnum.SUPER_ADMIN);
        return Optional.of(ds);
    }

    @Override
    public Optional<DataScopeDTO> findDataFieldProperty(Long userId) {
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        //所有 丽水市的code
        Collection<String> regionCodes = regionCacheHelper.listChildRegionCodeList(RegionConst.RC_LS,
                RegionConst.RL_CITY);

        DataScopeDTO ds = new DataScopeDTO();
        ds.setRegionCodes(new ArrayList<>(regionCodes));
        ds.setUserId(userFullInfo.getUserId());
        ds.setEmployeeCode(userFullInfo.getEmployeeCode());
        ds.setRole(RoleEnum.SUPER_ADMIN);
        return Optional.of(ds);
    }
}
