package com.hz.pm.api.expert.manage;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.constant.BizConst;
import com.hz.pm.api.common.model.FileBasicInfo;
import com.hz.pm.api.common.model.entity.KeyValDTO;
import com.hz.pm.api.common.util.BizUtils;
import com.hz.pm.api.expert.assembler.ExpertInfoCmdAssembler;
import com.hz.pm.api.expert.assembler.ExpertUserInfoAssembler;
import com.hz.pm.api.expert.constant.ExpertAccountStatusEnum;
import com.hz.pm.api.expert.constant.ExpertUserInfoStepEnum;
import com.hz.pm.api.expert.entity.ExpertUserFullInfo;
import com.hz.pm.api.expert.helper.ExpertInfoCommonHelper;
import com.hz.pm.api.expert.helper.ExpertManageHelper;
import com.hz.pm.api.expert.model.*;
import com.hz.pm.api.expert.model.cmd.ExpertFullInfoSaveCmd;
import com.hz.pm.api.expert.model.cmd.ExpertRecommendProofSaveCmd;
import com.hz.pm.api.expert.model.dto.ExpertDictionaryDTO;
import com.hz.pm.api.expert.model.dto.ExpertFullInfoAllDTO;
import com.hz.pm.api.expert.model.req.ExpertRegistrationRequest;
import com.hz.pm.api.expert.model.req.ExpertUserBasicInfoSubmitRequest;
import com.hz.pm.api.expert.model.vo.ExpertFullInfoVO;
import com.hz.pm.api.expert.model.vo.ExpertPortraitVO;
import com.hz.pm.api.expert.service.ExpertInfoService;
import com.hz.pm.api.expert.service.IExpertUserFullInfoService;
import com.hz.pm.api.meeting.entity.config.WebProperties;
import com.hz.pm.api.meeting.entity.domain.MeetingExpertJudge;
import com.hz.pm.api.meeting.entity.dto.ExpertInvitedRecordDTO;
import com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum;
import com.hz.pm.api.meeting.entity.enumeration.ExpertJudgeEnum.AttendStatus;
import com.hz.pm.api.meeting.entity.enumeration.ExpertJudgeEnum.Performance;
import com.hz.pm.api.meeting.service.IMeetingExpertJudgeService;
import com.hz.pm.api.meeting.service.IMeetingExpertService;
import com.hz.pm.api.meta.constant.ExpertDictTypeEnum;
import com.hz.pm.api.meta.model.ExpertRegionInfo;
import com.hz.pm.api.sms.constant.VerificationCodeType;
import com.hz.pm.api.sms.helper.VerifyCodeCheckHelper;
import com.hz.pm.api.sys.model.entity.Role;
import com.hz.pm.api.sys.model.entity.UserRole;
import com.hz.pm.api.sys.service.IRoleService;
import com.hz.pm.api.sys.service.IUserRoleService;
import com.hz.pm.api.user.constant.UserAvailableEnum;
import com.hz.pm.api.user.entity.UserInfo;
import com.hz.pm.api.user.entity.enumeration.RoleEnum;
import com.hz.pm.api.user.service.IUserInfoService;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.model.GenericResult;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.cache.model.cache.CacheKey;
import com.ningdatech.cache.repository.CachePlusOps;
import com.ningdatech.file.entity.vo.result.AttachFileVo;
import com.ningdatech.file.service.FileService;
import com.ningdatech.zwdd.client.ZwddClient;
import com.ningdatech.zwdd.model.dto.DingInfoByMobileDTO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.*;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @author liuxinxin
 * @date 2023/2/23 上午8:55
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class ExpertManage {

    private final ExpertManageHelper expertManageHelper;
    private final IExpertUserFullInfoService expertUserFullInfoService;
    private final ExpertInfoService expertInfoService;
    private final ExpertInfoCommonHelper expertInfoCommonHelper;
    private final FileService fileService;
    private final ExpertUserInfoAssembler expertUserInfoAssembler;
    private final IUserInfoService userInfoService;
    private final ExpertMetaApplyManage expertMetaApplyManage;
    private final IUserRoleService userRoleService;
    private final IRoleService roleService;
    private final VerifyCodeCheckHelper verifyCodeCheckHelper;
    private final CachePlusOps cachePlusOps;
    private final ZwddClient zwddClient;
    private final IMeetingExpertService meetingExpertService;
    private final IMeetingExpertJudgeService expertJudgeService;

    @Value("${expert-registration.verify-code.check:true}")
    private Boolean expertRegistrationVerifyCodeCheck;

    private static final Duration REGISTER_EXPIRED_DURATION = Duration.ofDays(3);
    private static final Duration REGISTER_GEN_DURATION = Duration.ofMinutes(30);

    private static final String RK_REGISTER_UNIQUE_ID = "expert_registration_id:";
    private static final String RK_REGISTER_UNIQUE_ID_LAST = RK_REGISTER_UNIQUE_ID + "last";

    public synchronized String getRegistrationUrl() {
        CacheKey lastKey = new CacheKey(RK_REGISTER_UNIQUE_ID_LAST, REGISTER_GEN_DURATION);
        String lastUniqueId = cachePlusOps.get(lastKey);
        if (StrUtil.isBlank(lastUniqueId)) {
            lastUniqueId = BizUtils.uuid32();
            CacheKey key = new CacheKey();
            key.setKey(RK_REGISTER_UNIQUE_ID + lastUniqueId);
            key.setExpire(REGISTER_EXPIRED_DURATION);
            cachePlusOps.set(lastKey, lastUniqueId);
            String gmtUserTime = LoginUserUtil.getUserId() + "#" + System.currentTimeMillis();
            cachePlusOps.set(key, gmtUserTime);
        }
        return WebProperties.webUrl + "/pm/expert/ephemeral/" + lastUniqueId + "/registration";
    }

    public void redirectToRegistrationUrl(String uniqueId, HttpServletResponse response) throws IOException {
        CacheKey cacheKey = new CacheKey(RK_REGISTER_UNIQUE_ID + uniqueId);
        response.setContentType(MediaType.TEXT_PLAIN_VALUE);
        response.setCharacterEncoding(StandardCharsets.UTF_8.name());
        try {
            if (cachePlusOps.exists(cacheKey)) {
                response.sendRedirect(WebProperties.webUrl + WebProperties.expertRegistrationUrl);
            } else {
                response.getWriter().write("专家报名链接已失效");
            }
        } catch (Exception e) {
            log.error("专家报名链接重定向异常：", e);
            response.getWriter().write("专家报名链接访问异常");
        }
    }


    /**
     * 专家管理员新增专家
     *
     * @param req /
     */
    @Transactional(rollbackFor = Exception.class)
    public void expertBasicInfoSubmit(ExpertUserBasicInfoSubmitRequest req) {
        Long operatorId = LoginUserUtil.getUserId();
        // 用户id
        Long userId = assemblerAndGenerateExpert(req, operatorId);
        // 批量通过专家入库审核/专家履职意向审核
        expertMetaApplyManage.autoPassExpertApply(userId);

        // 增加用户专家角色
        Role role = roleService.getOne(Wrappers.lambdaQuery(Role.class).eq(Role::getCode, RoleEnum.EXPERT.name()));
        UserRole expertRole = userRoleService.getOne(userId, role.getId());
        if (Objects.isNull(expertRole)) {
            expertRole = new UserRole();
            expertRole.setUserId(userId);
            expertRole.setRoleId(role.getId());
            userRoleService.save(expertRole);
        }
    }

    private void expertRecommendProofSubmit(List<DictionaryFieldInfo> recommendedWay, List<FileBasicInfo> recommendProofFile, Long expertUserId) {
        // 用户id
        ExpertUserFullInfo expertUserFullInfo = expertUserFullInfoService.getByUserId(expertUserId);
        // 判断专家状态，是否可以进行证明材料提交
        if (Objects.isNull(expertUserFullInfo)
                || !ExpertAccountStatusEnum.APPLYING.getKey().equals(expertUserFullInfo.getExpertAccountStatus())
                || ExpertUserInfoStepEnum.EVIDENCE_HAS_BEEN_SUBMITTED.getKey().equals(expertUserFullInfo.getUserInfoStep())) {
            throw new BizException("当前状态无法提交证明材料");
        }

        // 证明材料提交
        ExpertRecommendProofSaveCmd expertRecommendProofSaveCmd = new ExpertRecommendProofSaveCmd();
        if (CollectionUtils.isNotEmpty(recommendProofFile)) {
            expertRecommendProofSaveCmd.setRecommendationProofFileIdList(recommendProofFile.stream().map(FileBasicInfo::getFileId).collect(Collectors.toList()));
        }
        List<ExpertDictionaryDTO> recommendedWayDictionaryDTOList = new ArrayList<>();
        if (CollUtil.isNotEmpty(recommendedWay)) {
            recommendedWay.forEach(r -> {
                ExpertDictionaryDTO expertDictionaryDTO = new ExpertDictionaryDTO();
                expertDictionaryDTO.setDictionaryCode(r.getDictionaryCode());
                expertDictionaryDTO.setExpertInfoField(ExpertDictTypeEnum.RECOMMENDED_WAY.getKey());
                recommendedWayDictionaryDTOList.add(expertDictionaryDTO);
            });
        }
        expertRecommendProofSaveCmd.setRecommendedWay(recommendedWayDictionaryDTOList);
        expertRecommendProofSaveCmd.setUserId(expertUserId);
        expertInfoService.expertRecommendProofSave(expertRecommendProofSaveCmd);
    }

    public ExpertFullInfoVO getExpertFullInfoDetail(Long expertUserId) {
        Long currentUserId = LoginUserUtil.getUserId();
        if (Objects.isNull(expertUserId)) {
            expertUserId = currentUserId;
        }
        ExpertFullInfoAllDTO expertUserFullInfoAll = expertInfoService.getExpertUserFullInfoAll(expertUserId);
        if (Objects.isNull(expertUserFullInfoAll)) {
            return null;
        }
        List<Long> fileIdList = expertInfoCommonHelper.getExpertFileIdList(expertUserFullInfoAll);
        List<AttachFileVo> attachFiles = fileService.getByIds(fileIdList);
        return expertUserInfoAssembler.buildExpertFullInfo(attachFiles, expertUserFullInfoAll);
    }

    public Long generateOrGetUserId(ExpertBasicInfo basicInfo, Long operatorId) {
        String phoneNo = basicInfo.getPhoneNo();
        UserInfo userInfo = userInfoService.getUserInfoByPhoneNo(phoneNo);
        if (Objects.isNull(userInfo)) {
            GenericResult<DingInfoByMobileDTO> dingInfoByMobile = zwddClient.getDingInfoByMobile(basicInfo.getPhoneNo());
            if (dingInfoByMobile.isSuccess()) {
                DingInfoByMobileDTO data = dingInfoByMobile.getData();
                String accountId = data.getAccountId();
                userInfo = userInfoService.getUserInfoByAccountId(Long.parseLong(accountId));
            }
            if (userInfo == null) {
                userInfo = UserInfo.builder()
                        .username(basicInfo.getName())
                        .realName(basicInfo.getName())
                        .available(UserAvailableEnum.DISABLE.name())
                        .mobile(phoneNo)
                        .createBy(operatorId)
                        .updateBy(operatorId)
                        .createOn(LocalDateTime.now())
                        .updateOn(LocalDateTime.now())
                        .build();
                userInfoService.save(userInfo);
            } else if (!StrUtil.equals(userInfo.getRealName(), basicInfo.getName())) {
                log.warn("相似账户为：{} ==> {}", basicInfo.getPhoneNo(), userInfo);
                throw BizException.wrap("已存在同手机号，但不同名的账号");
            } else if (StrUtil.isBlank(userInfo.getMobile())) {
                userInfo.setMobile(basicInfo.getPhoneNo());
                userInfoService.updateById(userInfo);
            }
        }
        ExpertUserFullInfo one = expertUserFullInfoService.getOne(Wrappers.lambdaQuery(ExpertUserFullInfo.class)
                .eq(ExpertUserFullInfo::getPhoneNo, phoneNo));
        if (Objects.nonNull(one) && !one.getUserId().equals(userInfo.getId())) {
            throw new BizException("该手机号已被专家注册，请确认后再填写");
        }
        return userInfo.getId();
    }

    /**
     * 社会专家报名 需要校验手机号
     *
     * @param request \
     */
    @Transactional(rollbackFor = Exception.class)
    public void expertRegistration(ExpertRegistrationRequest request) {
        String verificationCode = request.getVerificationCode();
        String phoneNo = request.getBasicInfo().getPhoneNo();
        //校验 专家验证码
        if (Boolean.TRUE.equals(expertRegistrationVerifyCodeCheck)) {
            verifyCodeCheckHelper.verification(VerificationCodeType.EXPERT_REGISTER, phoneNo, verificationCode);
        }
        assemblerAndGenerateExpert(request, null);
    }


    public Long assemblerAndGenerateExpert(ExpertUserBasicInfoSubmitRequest req, Long operatorId) {
        if (Objects.isNull(operatorId)) {
            operatorId = -1L;
        }
        ExpertBasicInfo basicInfo = req.getBasicInfo();
        Long userId = generateOrGetUserId(basicInfo, operatorId);

        // 校验区域编码合法性 校验履职意向编码合法性
        ExpertRegionInfo expertRegionInfo = basicInfo.getExpertRegionInfo();
        expertManageHelper.expertRegionInfoCheck(expertRegionInfo);
        List<ExpertRegionInfo> expertIntentionWorkRegions = basicInfo.getExpertIntentionWorkRegions();
        for (ExpertRegionInfo expertIntentionWorkRegion : expertIntentionWorkRegions) {
            expertManageHelper.expertRegionInfoCheck(expertIntentionWorkRegion);
        }
        ExpertEduInfo eduInfo = req.getEduInfo();
        ExpertJobInfo jobInfo = req.getJobInfo();
        ExpertProfessionalInfo professionalInfo = req.getProfessionalInfo();
        // 校验标签字段
        expertManageHelper.tagFieldCheck(professionalInfo, basicInfo);
        // 校验字典字段
        expertManageHelper.dictionaryFieldCheck(basicInfo, eduInfo, jobInfo);
        // 判断专家提交状态，判断是否可以进行此操作
        ExpertUserFullInfo expertUserFullInfo = expertUserFullInfoService.getByUserId(userId);

        boolean submitBasicInfoStatusEnable = Objects.isNull(expertUserFullInfo)
                || (ExpertAccountStatusEnum.APPLYING.getKey().equals(expertUserFullInfo.getExpertAccountStatus())
                && !ExpertUserInfoStepEnum.EVIDENCE_HAS_BEEN_SUBMITTED.getKey().equals(expertUserFullInfo.getUserInfoStep()));
        if (submitBasicInfoStatusEnable) {
            // 新建 保存
            ExpertFullInfoSaveCmd expertFullInfoSaveCmd = ExpertInfoCmdAssembler
                    .buildExpertFullInfoSaveCmd(userId, basicInfo, eduInfo, jobInfo, professionalInfo);
            expertInfoService.saveExpertInfo(expertFullInfoSaveCmd);
        }

        ExpertRecommendInfo recommendInfo = req.getRecommendInfo();
        // 推荐证明材料
        List<DictionaryFieldInfo> recommendedWay = recommendInfo.getRecommendedWay();
        // 推荐方式
        List<FileBasicInfo> recommendProofFile = recommendInfo.getRecommendationProofFile();
        expertRecommendProofSubmit(recommendedWay, recommendProofFile, userId);
        return userId;
    }

    public ExpertPortraitVO expertPortrait(Long expertId, Integer year) {
        ExpertPortraitVO portrait = ExpertPortraitVO.init();
        LocalDate firstDate = LocalDate.of(year, 1, 1);
        LocalDateTime sTime = firstDate.atStartOfDay();
        LocalDateTime eTime = firstDate.with(TemporalAdjusters.lastDayOfYear()).atTime(LocalTime.MAX);
        List<ExpertInvitedRecordDTO> meetings = meetingExpertService.listByExpertIdAndTime(expertId, sTime, eTime);
        portrait.setReceivedCount(meetings.size());
        if (!meetings.isEmpty()) {
            AtomicInteger agreedCount = new AtomicInteger(0);
            Map<Integer, Long> countByPerMonth = meetings.stream()
                    .filter(w -> {
                        boolean agreed = ExpertAttendStatusEnum.AGREED.eq(w.getStatus());
                        if (agreed) {
                            agreedCount.incrementAndGet();
                        }
                        return agreed;
                    }).collect(Collectors.groupingBy(w -> w.getCreateOn().getMonthValue(), Collectors.counting()));
            portrait.setAttendCountPerMonth(new ArrayList<>());
            for (int i = 1; i <= Month.DECEMBER.getValue(); i++) {
                KeyValDTO<Integer, Long> currMonth = KeyValDTO.of(i, countByPerMonth.getOrDefault(i, 0L));
                portrait.getAttendCountPerMonth().add(currMonth);
            }
            portrait.setConfirmedCount(agreedCount.get());
            List<Long> meetingExpertIds = CollUtils.fieldList(meetings, ExpertInvitedRecordDTO::getMeetingExpertId);
            List<MeetingExpertJudge> expertJudges = expertJudgeService.listByMeetingExpertIds(meetingExpertIds);
            if (!expertJudges.isEmpty()) {
                int positivity = 0;
                for (MeetingExpertJudge expertJudge : expertJudges) {
                    if (AttendStatus.BE_LATE.eq(expertJudge.getAttended())) {
                        portrait.incrBeLateCount();
                    }
                    if (Boolean.TRUE.equals(expertJudge.getLeaveEarly())) {
                        portrait.incrLeaveEarlyCount();
                    }
                    if (Boolean.TRUE.equals(expertJudge.getBrokeRule())) {
                        portrait.incrViolationCount();
                    }
                    if (Performance.POSITIVE.eq(expertJudge.getPerformance())) {
                        positivity++;
                    }
                }
                portrait.setPositivity((int) Math.round(positivity * BizConst.D100 / expertJudges.size()));
            }

        }
        return portrait;
    }

}
