package com.hz.pm.api.external;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.codec.Base64Encoder;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.core.map.MapUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.hz.pm.api.external.model.dto.MhExpertDTO;
import com.hz.pm.api.external.model.dto.MhOrgDTO;
import com.hz.pm.api.external.model.dto.MhRetDTO;
import com.hz.pm.api.external.model.dto.MhUserDTO;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Component;

import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.hutool.core.date.DatePattern.NORM_DATETIME_FORMATTER;

/**
 * <p>
 * UserOrgClient
 * </p>
 *
 * @author WendyYang
 * @since 20:41 2023/12/20
 */
@Component
public class MhUserOrgClient {

    @Value("${xc.api-host:}")
    private String xcApiHost;

    public static final String UNIT_URL = "/sync/unit";
    public static final String USER_URL = "/sync/user";
    public static final String EXPERT_URL = "/sync/expert";

    public MhRetDTO<List<MhOrgDTO>> queryOrges() {
        String requestUrl = xcApiHost + UNIT_URL;
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhOrgDTO>>>() {
        }, false);
    }

    public MhRetDTO<List<MhUserDTO>> queryUsers(LocalDateTime syncDateTime) {
        String requestUrl = xcApiHost + USER_URL;
        if (syncDateTime != null) {
            String timeString = LocalDateTimeUtil.format(syncDateTime, NORM_DATETIME_FORMATTER);
            requestUrl = "?syncDateTime=" + timeString;
        }
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<List<MhUserDTO>>>() {
        }, false);
    }

    public MhRetDTO<MhExpertDTO> queryExperts(LocalDateTime syncDateTime) {
        String requestUrl = xcApiHost + EXPERT_URL;
        if (syncDateTime != null) {
            String timeString = LocalDateTimeUtil.format(syncDateTime, NORM_DATETIME_FORMATTER);
            requestUrl = "?syncDateTime=" + timeString;
        }
        String retBody = HttpUtil.get(requestUrl);
        return JSONUtil.toBean(retBody, new TypeReference<MhRetDTO<MhExpertDTO>>() {
        }, false);
    }

}
