package com.hz.pm.api.meta.helper.basic;

import com.github.benmanes.caffeine.cache.Caffeine;
import com.github.benmanes.caffeine.cache.LoadingCache;
import com.hz.pm.api.meta.constant.DictTypeEnum;
import com.hz.pm.api.meta.helper.DictionaryCache;
import com.hz.pm.api.meta.model.dto.DictDTO;
import com.hz.pm.api.meta.service.IMetaDictionaryService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.InitializingBean;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * @author liuxinxin
 * @date 2022/12/22 上午11:03
 */
@RequiredArgsConstructor
public abstract class AbstractDictionaryCache implements InitializingBean, DictionaryCache {

    private final IMetaDictionaryService metaDictionaryService;

    protected Map<String, DictDTO> dictionaryMap = new ConcurrentHashMap<>();
    protected LoadingCache<String, List<DictDTO>> dictCache;

    @Override
    public void afterPropertiesSet() {
        dictCache = Caffeine.newBuilder()
                .refreshAfterWrite(30, TimeUnit.MINUTES)
                .build(key -> {
                    List<DictDTO> dicts = metaDictionaryService.queryAll();
                    dictionaryMap = buildCateMap(dicts);
                    Map<String, List<DictDTO>> dictMapByType = CollUtils.group(dicts, DictDTO::getDictionaryType);
                    return dictMapByType.get(key);
                });
        warmUp();
    }

    private Map<String, DictDTO> buildCateMap(List<DictDTO> dictionaryDTOList) {
        Map<String, DictDTO> dictionaryDtoMap = new ConcurrentHashMap<>(128);
        dictionaryDTOList.forEach(dictionaryDTO -> dictionaryDtoMap.put(dictionaryDTO.getDictionaryCode(), dictionaryDTO));
        return dictionaryDtoMap;
    }

    private void warmUp() {
        for (DictTypeEnum typeEnum : DictTypeEnum.values()) {
            dictCache.get(typeEnum.getKey());
        }
    }

}
