package com.hz.pm.api.user.security.auth.agent;

import com.hz.pm.api.user.security.auth.AuthProperties;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.SecurityConfigurerAdapter;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.web.DefaultSecurityFilterChain;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.stereotype.Component;

/**
 * <p>
 * AgentAuthSecurityConfig
 * </p>
 *
 * @author WendyYang
 * @since 14:24 2023/12/16
 */
@Component
public class AgentAuthSecurityConfig extends SecurityConfigurerAdapter<DefaultSecurityFilterChain, HttpSecurity> {


    protected final AuthenticationSuccessHandler defaultLoginSuccessHandler;
    protected final AuthenticationFailureHandler defaultLoginFailureHandler;
    private final UserDetailsService agentLoginUserDetailService;
    private final AuthProperties authProperties;

    public AgentAuthSecurityConfig(@Qualifier(value = "defaultLoginSuccessHandler") AuthenticationSuccessHandler loginSuccessHandler,
                                   @Qualifier(value = "defaultLoginFailureHandler") AuthenticationFailureHandler loginFailureHandler,
                                   @Qualifier(value = "agentLoginUserDetailService") UserDetailsService agentLoginUserDetailService,
                                   AuthProperties authProperties) {
        this.defaultLoginSuccessHandler = loginSuccessHandler;
        this.defaultLoginFailureHandler = loginFailureHandler;
        this.agentLoginUserDetailService = agentLoginUserDetailService;
        this.authProperties = authProperties;
    }

    @Override
    public void configure(HttpSecurity http) {
        AgentAuthFilter agentAuthFilter =
                new AgentAuthFilter(authProperties.getAgentLoginUrl());
        AuthenticationManager authenticationManager = http.getSharedObject(AuthenticationManager.class);
        agentAuthFilter.setAuthenticationManager(authenticationManager);
        agentAuthFilter.setAuthenticationSuccessHandler(defaultLoginSuccessHandler);
        agentAuthFilter.setAuthenticationFailureHandler(defaultLoginFailureHandler);

        AgentAuthProvider authenticationProvider = new AgentAuthProvider();
        authenticationProvider.setUserDetailsService(agentLoginUserDetailService);

        http.authenticationProvider(authenticationProvider)
                .addFilterAfter(agentAuthFilter, UsernamePasswordAuthenticationFilter.class);
    }

}
