package com.hz.pm.api.projectCollection;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.log.model.domain.OptLog;
import com.ningdatech.log.service.OptLogService;
import com.hz.pm.api.AppTests;
import com.hz.pm.api.common.constant.BizConst;
import com.hz.pm.api.gov.model.dto.GovBizProjectApplyDTO;
import com.hz.pm.api.gov.model.dto.GovBizProjectBaseinfoDTO;
import com.hz.pm.api.gov.model.dto.GovBizProjectSaveDTO;
import com.hz.pm.api.gov.model.entity.*;
import com.hz.pm.api.gov.service.*;
import com.hz.pm.api.portrait.model.entity.ProjectTag;
import com.hz.pm.api.portrait.service.IProjectTagService;
import com.hz.pm.api.projectdeclared.model.entity.Operation;
import com.hz.pm.api.projectdeclared.model.entity.PreInsAcceptancePerson;
import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.service.IOperationService;
import com.hz.pm.api.projectdeclared.service.IPreInsAcceptancePersonService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectdeclared.utils.GenerateProjectCodeUtil;
import com.hz.pm.api.projectlib.model.entity.*;
import com.hz.pm.api.projectlib.service.*;
import com.hz.pm.api.safety.model.entity.PersonSafetyInfo;
import com.hz.pm.api.safety.model.entity.SupplierSafetyQualification;
import com.hz.pm.api.safety.service.IPersonSafetyInfoService;
import com.hz.pm.api.safety.service.ISupplierSafetyQualificationService;
import com.hz.pm.api.scheduler.task.SynProjectCollectionTask;
import com.hz.pm.api.scheduler.task.SynProjectCoreBizTask;
import com.hz.pm.api.staging.model.entity.ProjectStaging;
import com.hz.pm.api.staging.service.IProjectStagingService;
import com.wflow.bean.entity.WflowCcTasks;
import com.wflow.service.WflowCcTasksService;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.HistoryService;
import org.flowable.engine.RuntimeService;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.net.UnknownHostException;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.time.temporal.Temporal;
import java.util.List;
import java.util.Objects;
import java.util.StringJoiner;
import java.util.stream.Collectors;

/**
 * @Classname ProjectCollectionTest
 * @Description
 * @Date 2023/9/7 15:08
 * @Author PoffyZhang
 */
public class ProjectRemoveTest extends AppTests {

    @Autowired
    private IProjectService projectService;

    @Autowired
    private IProjectApplicationService applicationService;

    @Autowired
    private IProjectInstService projectInstService;

    @Autowired
    private IProjectStagingService stagingService;

    @Autowired
    private INdProjectStatusChangeService statusChangeService;

    @Autowired
    private ISupplierSafetyQualificationService supplierSafetyQualificationService;

    @Autowired
    private IPersonSafetyInfoService personService;

    @Autowired
    private IOperationService operationService;

    @Autowired
    private IPurchaseService purchaseService;

    @Autowired
    private IProjectTagService projectTagService;

    @Autowired
    private IPreInsAcceptancePersonService preInsAcceptancePersonService;

    @Autowired
    private IProjectRenewalFundDeclarationService renewalFundDeclarationService;

    @Autowired
    private HistoryService historyService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private WflowCcTasksService ccTasksService;

    @Test
    public void test(){
        List<String> projectCodes = Lists.newArrayList("33110000020230100078","33110000020230100077");

        for(String projectCode : projectCodes){
            //项目
            List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                    .eq(Project::getProjectCode, projectCode));

            List<Long> projectIds = projects.stream().map(Project::getId).collect(Collectors.toList());

            //1.删除应用
            applicationService.remove(Wrappers.lambdaQuery(ProjectApplication.class)
                    .eq(ProjectApplication::getProjectCode,projectCode));

            //2.删除相关流程
            List<ProjectInst> projectInsts = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                    .in(ProjectInst::getProjectId, projectIds));
            for(ProjectInst projectInst : projectInsts){
                //实例
                try{
                    runtimeService.deleteProcessInstance(projectInst.getInstCode(),"删除");
                }catch (Exception e){

                }
                try{
                    historyService.deleteHistoricProcessInstance(projectInst.getInstCode());
                }catch (Exception e){

                }
                //抄送
                ccTasksService.remove(Wrappers.lambdaQuery(WflowCcTasks.class)
                        .eq(WflowCcTasks::getInstanceId,projectInst.getInstCode()));
            }
            projectInstService.removeByIds(projectInsts.stream().map(ProjectInst::getId).collect(Collectors.toList()));

            //3.删除实施表
            operationService.remove(Wrappers.lambdaQuery(Operation.class)
                    .eq(Operation::getProjectCode,projectCode));

            //4.删除续建项目资金表
            renewalFundDeclarationService.remove(Wrappers.lambdaQuery(ProjectRenewalFundDeclaration.class)
                    .eq(ProjectRenewalFundDeclaration::getProjectCode,projectCode));

            //5.暂存表
            stagingService.remove(Wrappers.lambdaQuery(ProjectStaging.class)
                    .in(ProjectStaging::getProjectId,projectIds));

            //6.状态变更表
            statusChangeService.remove(Wrappers.lambdaQuery(ProjectStatusChange.class)
                    .in(ProjectStatusChange::getProjectId,projectIds));

            //7.采购表
            purchaseService.remove(Wrappers.lambdaQuery(Purchase.class)
                    .in(Purchase::getProjectId,projectIds));

            //8.标签
            projectTagService.remove(Wrappers.lambdaQuery(ProjectTag.class)
                    .eq(ProjectTag::getProjectCode,projectCode));

            //9.初验人员
            preInsAcceptancePersonService.remove(Wrappers.lambdaQuery(PreInsAcceptancePerson.class)
                    .in(PreInsAcceptancePerson::getProjectId,projectIds));

            //10.供应商安全质量
            supplierSafetyQualificationService.remove(Wrappers.lambdaQuery(SupplierSafetyQualification.class)
                    .eq(SupplierSafetyQualification::getProjectCode,projectCode));

            //11.安全人员
            personService.remove(Wrappers.lambdaQuery(PersonSafetyInfo.class)
                    .eq(PersonSafetyInfo::getProjectCode,projectCode));

            //12.最后删除项目
            projectService.remove(Wrappers.lambdaQuery(Project.class)
                    .eq(Project::getProjectCode, projectCode));
        }
    }

}
